@@ -19,11 +19,10 @@ from pathlib import Path
 from typing import Any, Dict, Iterator, List, Optional, Type, cast
 
 import minizinc
-from minizinc.dzn import UnknownExpression
 from minizinc.error import parse_error
 from minizinc.instance import Instance
 from minizinc.json import MZNJSONEncoder
-from minizinc.model import Method, Model, ParPath
+from minizinc.model import Method, Model, ParPath, UnknownExpression
 from minizinc.result import Result, Status, parse_solution, set_stat
 from minizinc.solver import Solver
 
@@ -253,6 +252,7 @@ class CLIInstance(Instance):
         all_solutions=False,
         intermediate_solutions=False,
         free_search: bool = False,
+        optimisation_level: Optional[int] = None,
         ignore_errors=False,
         **kwargs,
     ):
@@ -311,6 +311,9 @@ class CLIInstance(Instance):
             if "-f" not in self._solver.stdFlags:
                 raise NotImplementedError("Solver does not support the -f flag")
             cmd.append("-f")
+        # Set compiler optimisation level if specified
+        if optimisation_level:
+            cmd.extend(["-O", str(optimisation_level)])
 
         # Set time limit for the MiniZinc solving
         if timeout is not None:
@@ -326,11 +329,13 @@ class CLIInstance(Instance):
                 cmd.extend([flag, value])
 
         # Add files as last arguments
-        with self.files() as files:
+        with self.files() as files, self._solver.configuration() as solver:
             assert self.output_type is not None
             cmd.extend(files)
             # Run the MiniZinc process
-            proc = await self._driver.create_process(cmd, solver=self._solver)
+            proc = await self._driver.create_process(cmd, solver=solver)
+            assert proc.stderr is not None
+            assert proc.stdout is not None
 
             status = Status.UNKNOWN
             code = 0
@@ -395,6 +400,7 @@ class CLIInstance(Instance):
         all_solutions=False,
         intermediate_solutions=False,
         free_search: bool = False,
+        optimisation_level: Optional[int] = None,
         **kwargs,
     ):
         status = Status.UNKNOWN
@@ -414,6 +420,7 @@ class CLIInstance(Instance):
             random_seed=random_seed,
             all_solutions=all_solutions,
             free_search=free_search,
+            optimisation_level=optimisation_level,
             **kwargs,
         ):
             status = result.status
@@ -426,13 +433,28 @@ class CLIInstance(Instance):
         return Result(status, solution, statistics)
 
     @contextlib.contextmanager
-    def flat(self, timeout: Optional[timedelta] = None, **kwargs):
+    def flat(
+        self,
+        timeout: Optional[timedelta] = None,
+        optimisation_level: Optional[int] = None,
+        **kwargs,
+    ):
         """Produce a FlatZinc file for the instance.
 
         Args:
             timeout (Optional[timedelta]): Set the time limit for the process
                 of flattening the instance. TODO: An exception is raised if the
                 timeout is reached.
+            optimisation_level (Optional[int]): Set the MiniZinc compiler
+                optimisation level.
+
+                - 0: Disable optimisation
+                - 1: Single pass optimisation (default)
+                - 2: Flatten twice to improve flattening decisions
+                - 3: Perform root-node-propagation
+                - 4: Probe bounds of all variables at the root node
+                - 5: Probe values of all variables at the root node
+
             **kwargs: Other flags to be passed to the compiler. ``--`` can be
                 omitted in the name of the flag. If the type of the flag is
                 Boolean, then its value signifies its occurrence.
@@ -451,6 +473,10 @@ class CLIInstance(Instance):
         cmd.extend(["--ozn", ozn.name])
         ozn.close()
 
+        # Set compiler optimisation level if specified
+        if optimisation_level:
+            cmd.extend(["-O", str(optimisation_level)])
+
         for flag, value in kwargs.items():
             if not flag.startswith("-"):
                 flag = "--" + flag
